@@ -4,6 +4,7 @@ import datasets
 from datasets import load_dataset, Dataset
 from transformers import DefaultDataCollator
 from transformers import DataCollatorForTokenClassification
+from transformers import PreTrainedTokenizerBase
 from evaluate import load as load_metric
 from lazy_load import lazy_func
 from easydict import EasyDict as edict
@@ -11,22 +12,24 @@ import funcy as fc
 import evaluate
 from dataclasses import dataclass, field
 import re
+from tokenizers.tokenization_utils_base import PreTrainedTokenizerBase
 
 load_dataset = lazy_func(datasets.load_dataset)
-_=None
+
 
 def get_name(dataset):
     try:
-        s=str(dataset.cache_files.values())
-        return re.search(r'/datasets/(.*?)/default/', s).group(1).split('___')[-1]
+        s = str(dataset.cache_files.values())
+        return re.search(r"/datasets/(.*?)/default/", s).group(1).split("___")[-1]
     except:
-        return ''
+        return ""
+
 
 @dataclass
 class Task:
-    dataset:Dataset=None
-    name:str=''
-    tokenizer:_=None
+    dataset: Dataset = None
+    name: str = ""
+    tokenizer: PreTrainedTokenizerBase = None
 
     def __hash__(self):
         return hash(str(self.dataset.__dict__))
@@ -35,30 +38,34 @@ class Task:
 
         self.__class__.__hash__ = Task.__hash__
         if type(self.dataset) == str:
-            name=self.dataset
+            name = self.dataset
             self.dataset = load_dataset(self.dataset)
         elif type(self.dataset) == tuple:
-            name="/".join(self.dataset)
+            name = "/".join(self.dataset)
             self.dataset = load_dataset(*self.dataset)
         else:
-            name=get_name(self.dataset)
-        
+            name = get_name(self.dataset)
+
         if not self.name:
-            self.name=name
+            self.name = name
 
+    def set_tokenizer(self, tokenizer):
+        self.tokenizer = tokenizer
 
-    def set_tokenizer(self,tokenizer):
-        self.tokenizer=tokenizer
 
 @dataclass
 class Classification(Task):
     task_type = "SequenceClassification"
-    dataset:Dataset=None
+    dataset: Dataset = None
     data_collator = DefaultDataCollator()
-    tokenizer_kwargs :_= field(default_factory=lambda:edict(truncation=True, padding="max_length", max_length=256))
-    s1:str='sentence1'
-    s2:str='sentence2'
-    y:str='labels'
+    tokenizer_kwargs: _ = field(
+        default_factory=lambda: edict(
+            truncation=True, padding="max_length", max_length=256
+        )
+    )
+    s1: str = "sentence1"
+    s2: str = "sentence2"
+    y: str = "labels"
     num_labels = None
 
     def __post_init__(self):
@@ -80,7 +87,7 @@ class Classification(Task):
     def compute_metrics(self, eval_pred):
         predictions, labels = eval_pred.predictions, eval_pred.label_ids
         if "int" in str(eval_pred.label_ids.dtype):
-            metric = load_metric("glue", "mnli")
+            metric = load_metric("super_glue", "cb")
             predictions = np.argmax(predictions, axis=1)
         else:
             metric = load_metric("glue", "stsb")
@@ -118,13 +125,15 @@ class DataCollatorForMultipleChoice:
 @dataclass
 class MultipleChoice(Classification):
     task_type = "MultipleChoice"
-    dataset:Dataset=None
-    tokenizer_kwargs :_= field(default_factory=lambda:edict(padding="max_length", max_length=256))
+    dataset: Dataset = None
+    tokenizer_kwargs: _ = field(
+        default_factory=lambda: edict(padding="max_length", max_length=256)
+    )
 
     num_labels = 2
 
-    choices:_=field(default_factory=list)
-    s1:str='inputs'
+    choices: _ = field(default_factory=list)
+    s1: str = "inputs"
 
     def set_tokenizer(self, tokenizer):
         self.tokenizer = tokenizer
@@ -156,15 +165,19 @@ class MultipleChoice(Classification):
         }
         return outputs
 
+
 @dataclass
 class TokenClassification(Task):
     task_type = "TokenClassification"
-    dataset:Dataset=None
+    dataset: Dataset = None
     metric = evaluate.load("seqeval")
-    tokenizer_kwargs:_=field(default_factory=lambda:edict(truncation=True, padding="max_length", max_length=256))
-    tokens:str=None
-    y:str=None
-
+    tokenizer_kwargs: _ = field(
+        default_factory=lambda: edict(
+            truncation=True, padding="max_length", max_length=256
+        )
+    )
+    tokens: str = None
+    y: str = None
 
     @staticmethod
     def align_labels_with_tokens(labels, word_ids):
@@ -194,10 +207,12 @@ class TokenClassification(Task):
         self.num_labels = 1 if "float" in target.dtype else target.feature.num_classes
         self.label_names = [f"{i}" for i in range(self.num_labels)]
 
-    def set_tokenizer(self,tokenizer):
-        self.tokenizer=tokenizer
-        self.tokenizer.add_prefix_space=True
-        self.data_collator = DataCollatorForTokenClassification(tokenizer=self.tokenizer)
+    def set_tokenizer(self, tokenizer):
+        self.tokenizer = tokenizer
+        self.tokenizer.add_prefix_space = True
+        self.data_collator = DataCollatorForTokenClassification(
+            tokenizer=self.tokenizer
+        )
 
     def preprocess_function(self, examples):
         tokenized_inputs = self.tokenizer(
@@ -227,9 +242,12 @@ class TokenClassification(Task):
         all_metrics = self.metric.compute(
             predictions=true_predictions, references=true_labels
         )
+        meta = {"name": self.name, "size": len(predictions), "index": self.index}
+
         return {
             "precision": all_metrics["overall_precision"],
             "recall": all_metrics["overall_recall"],
             "f1": all_metrics["overall_f1"],
             "accuracy": all_metrics["overall_accuracy"],
+            **meta,
         }
